@@ -11,15 +11,20 @@
 #include <qtsingleapplication.h>
 
 #include <utils/algorithm.h>
+#include <utils/appinfo.h>
+#include <utils/aspects.h>
 #include <utils/environment.h>
 #include <utils/fileutils.h>
 #include <utils/fsengine/fsengine.h>
 #include <utils/hostosinfo.h>
+#include <utils/plaintextedit/plaintexteditaccessibility.h>
+#include <utils/processreaper.h>
 #include <utils/qtcsettings.h>
-#include <utils/singleton.h>
+#include <utils/stylehelper.h>
 #include <utils/temporarydirectory.h>
 #include <utils/terminalcommand.h>
 
+#include <QAccessible>
 #include <QDebug>
 #include <QDir>
 #include <QFileInfo>
@@ -32,6 +37,7 @@
 #include <QScopeGuard>
 #include <QStandardPaths>
 #include <QStyle>
+#include <QSurfaceFormat>
 #include <QTextCodec>
 #include <QTextStream>
 #include <QThreadPool>
@@ -42,28 +48,33 @@
 #include <string>
 #include <vector>
 
-#ifdef ENABLE_QT_BREAKPAD
-#include <qtsystemexceptionhandler.h>
-#endif
-
 #ifdef ENABLE_CRASHPAD
 #define NOMINMAX
-#include "client/crashpad_client.h"
 #include "client/crash_report_database.h"
+#include "client/crashpad_client.h"
+#include "client/crashpad_info.h"
 #include "client/settings.h"
 #endif
 
+#ifdef ENABLE_SENTRY
+#include <sentry.h>
+
+Q_LOGGING_CATEGORY(sentryLog, "qtc.sentry", QtWarningMsg)
+#endif
+
 using namespace ExtensionSystem;
+using namespace Utils;
 
 enum { OptionIndent = 4, DescriptionIndent = 34 };
 
-const char corePluginNameC[] = "Core";
+const char corePluginIdC[] = "core";
 const char fixedOptionsC[]
     = " [OPTION]... [FILE]...\n"
       "Options:\n"
       "    -help                         Display this help\n"
       "    -version                      Display program version\n"
       "    -client                       Attempt to connect to already running first instance\n"
+      "    -clientid                     A postfix for the ID used by -client\n"
       "    -settingspath <path>          Override the default path where user settings are stored\n"
       "    -installsettingspath <path>   Override the default path from where user-independent "
       "settings are read\n"
@@ -80,9 +91,12 @@ const char HELP_OPTION4[] = "--help";
 const char VERSION_OPTION[] = "-version";
 const char VERSION_OPTION2[] = "--version";
 const char CLIENT_OPTION[] = "-client";
+const char CLIENTID_OPTION[] = "-clientid";
 const char SETTINGS_OPTION[] = "-settingspath";
 const char INSTALL_SETTINGS_OPTION[] = "-installsettingspath";
 const char TEST_OPTION[] = "-test";
+const char STYLE_OPTION[] = "-style";
+const char QML_LITE_DESIGNER_OPTION[] = "-qml-lite-designer";
 const char TEMPORARY_CLEAN_SETTINGS1[] = "-temporarycleansettings";
 const char TEMPORARY_CLEAN_SETTINGS2[] = "-tcs";
 const char PID_OPTION[] = "-pid";
@@ -91,8 +105,6 @@ const char PLUGINPATH_OPTION[] = "-pluginpath";
 const char LANGUAGE_OPTION[] = "-language";
 const char USER_LIBRARY_PATH_OPTION[] = "-user-library-path"; // hidden option for qtcreator.sh
 
-using PluginSpecSet = QVector<PluginSpec *>;
-
 // Helpers for displaying messages. Note that there is no console on Windows.
 
 // Format as <pre> HTML
@@ -109,7 +121,7 @@ static inline QString toHtml(const QString &t)
 
 static void displayHelpText(const QString &t)
 {
-    if (Utils::HostOsInfo::isWindowsHost() && qApp)
+    if (HostOsInfo::isWindowsHost() && qApp)
         QMessageBox::information(nullptr, QLatin1String(Core::Constants::IDE_DISPLAY_NAME), toHtml(t));
     else
         printf("%s", qPrintable(t));
@@ -117,7 +129,7 @@ static void displayHelpText(const QString &t)
 
 static void displayError(const QString &t)
 {
-    if (Utils::HostOsInfo::isWindowsHost() && qApp)
+    if (HostOsInfo::isWindowsHost() && qApp)
         QMessageBox::critical(nullptr, QLatin1String(Core::Constants::IDE_DISPLAY_NAME), t);
     else
         qCritical("%s", qPrintable(t));
@@ -177,34 +189,30 @@ static inline int askMsgSendFailed()
                 QMessageBox::Retry);
 }
 
-static inline QStringList getPluginPaths()
+static inline FilePaths getPluginPaths()
 {
-    QStringList rc(QDir::cleanPath(QApplication::applicationDirPath()
-                                   + '/' + RELATIVE_PLUGIN_PATH));
+    FilePaths rc;
+    rc << appInfo().plugins << appInfo().luaPlugins << appInfo().userLuaPlugins;
+
+    const auto version = [](int micro) {
+        return QString("%1.%2.%3").arg(IDE_VERSION_MAJOR).arg(IDE_VERSION_MINOR).arg(micro);
+    };
+
+    const int minPatchVersion = qMin(
+        IDE_VERSION_RELEASE,
+        QVersionNumber::fromString(Core::Constants::IDE_VERSION_COMPAT).microVersion());
+
     // Local plugin path: <localappdata>/plugins/<ideversion>
     //    where <localappdata> is e.g.
     //    "%LOCALAPPDATA%\QtProject\qtcreator" on Windows Vista and later
     //    "$XDG_DATA_HOME/data/QtProject/qtcreator" or "~/.local/share/data/QtProject/qtcreator" on Linux
     //    "~/Library/Application Support/QtProject/Qt Creator" on Mac
-    QString pluginPath = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
-    if (Utils::HostOsInfo::isAnyUnixHost() && !Utils::HostOsInfo::isMacHost())
-        pluginPath += QLatin1String("/data");
-    pluginPath += QLatin1Char('/')
-            + QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR)
-            + QLatin1Char('/');
-    pluginPath += QLatin1String(Utils::HostOsInfo::isMacHost() ?
-                                    Core::Constants::IDE_DISPLAY_NAME :
-                                    Core::Constants::IDE_ID);
-    pluginPath += QLatin1String("/plugins/");
+    const FilePath userPluginPath = appInfo().userPluginsRoot.parentDir();
+
     // Qt Creator X.Y.Z can load plugins from X.Y.(Z-1) etc, so add current and previous
     // patch versions
-    const QString minorVersion = QString::number(IDE_VERSION_MAJOR) + '.'
-                                 + QString::number(IDE_VERSION_MINOR) + '.';
-    const int minPatchVersion
-        = qMin(IDE_VERSION_RELEASE,
-               QVersionNumber::fromString(Core::Constants::IDE_VERSION_COMPAT).microVersion());
     for (int patchVersion = IDE_VERSION_RELEASE; patchVersion >= minPatchVersion; --patchVersion)
-        rc.push_back(pluginPath + minorVersion + QString::number(patchVersion));
+        rc << userPluginPath / version(patchVersion);
     return rc;
 }
 
@@ -268,26 +276,46 @@ static void setupInstallSettings(QString &installSettingspath, bool redirect = t
     } while (containsInstallSettingsKey && count < 3);
 }
 
-static Utils::QtcSettings *createUserSettings()
+static void setupAccessibility()
 {
-    return new Utils::QtcSettings(QSettings::IniFormat,
-                                  QSettings::UserScope,
-                                  QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
-                                  QLatin1String(Core::Constants::IDE_CASED_ID));
+    QAccessible::installFactory(&accessiblePlainTextEditFactory);
+}
+
+static QtcSettings *createUserSettings()
+{
+    return new QtcSettings(QSettings::IniFormat,
+                           QSettings::UserScope,
+                           QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
+                           QLatin1String(Core::Constants::IDE_CASED_ID));
 }
 
 static void setHighDpiEnvironmentVariable()
 {
-    if (Utils::HostOsInfo::isMacHost() || qEnvironmentVariableIsSet("QT_SCALE_FACTOR_ROUNDING_POLICY"))
+    if (StyleHelper::defaultHighDpiScaleFactorRoundingPolicy()
+            == Qt::HighDpiScaleFactorRoundingPolicy::Unset
+        || qEnvironmentVariableIsSet(StyleHelper::C_QT_SCALE_FACTOR_ROUNDING_POLICY))
         return;
 
-    std::unique_ptr<QSettings> settings(createUserSettings());
+    std::unique_ptr<QtcSettings> settings(createUserSettings());
 
-    const bool defaultValue = Utils::HostOsInfo::isWindowsHost();
-    const bool enableHighDpiScaling = settings->value("Core/EnableHighDpiScaling", defaultValue).toBool();
-    const auto policy = enableHighDpiScaling ? Qt::HighDpiScaleFactorRoundingPolicy::PassThrough
-                                             : Qt::HighDpiScaleFactorRoundingPolicy::Floor;
-    QGuiApplication::setHighDpiScaleFactorRoundingPolicy(policy);
+    using Policy = Qt::HighDpiScaleFactorRoundingPolicy;
+    const Policy defaultPolicy = StyleHelper::defaultHighDpiScaleFactorRoundingPolicy();
+    const Policy userPolicy = settings->value("Core/HighDpiScaleFactorRoundingPolicy",
+                                              int(defaultPolicy)).value<Policy>();
+    QGuiApplication::setHighDpiScaleFactorRoundingPolicy(userPolicy);
+}
+
+static void setRHIOpenGLVariable()
+{
+    QSettings installSettings(
+        QSettings::IniFormat,
+        QSettings::SystemScope,
+        QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
+        QLatin1String(Core::Constants::IDE_CASED_ID));
+
+    const QVariant value = installSettings.value("Core/RhiBackend");
+    if (value.isValid())
+        qputenv("QSG_RHI_BACKEND", value.toByteArray());
 }
 
 void setPixmapCacheLimit()
@@ -313,6 +341,7 @@ struct Options
     QString installSettingsPath;
     QStringList customPluginPaths;
     QString uiLanguage;
+    QString singleAppIdPostfix;
     // list of arguments that were handled and not passed to the application or plugin manager
     QStringList preAppArguments;
     // list of arguments to be passed to the application or plugin manager
@@ -320,6 +349,7 @@ struct Options
     std::optional<QString> userLibraryPath;
     bool hasTestOption = false;
     bool wantsCleanSettings = false;
+    bool hasStyleOption = false;
 };
 
 Options parseCommandLine(int argc, char *argv[])
@@ -355,13 +385,22 @@ Options parseCommandLine(int argc, char *argv[])
         } else if (arg == TEMPORARY_CLEAN_SETTINGS1 || arg == TEMPORARY_CLEAN_SETTINGS2) {
             options.wantsCleanSettings = true;
             options.preAppArguments << arg;
+        } else if (arg == CLIENTID_OPTION && hasNext) {
+            ++it;
+            options.singleAppIdPostfix = nextArg;
+            options.preAppArguments << arg << nextArg;
         } else { // arguments that are still passed on to the application
+            if (arg == STYLE_OPTION)
+                options.hasStyleOption = true;
             if (arg == TEST_OPTION)
                 options.hasTestOption = true;
+            if (arg == QML_LITE_DESIGNER_OPTION)
+                options.singleAppIdPostfix = QML_LITE_DESIGNER_OPTION;
             options.appArguments.push_back(*it);
         }
         ++it;
     }
+
     return options;
 }
 
@@ -401,40 +440,27 @@ private:
 QStringList lastSessionArgument()
 {
     // using insider information here is not particularly beautiful, anyhow
-    const bool hasProjectExplorer = Utils::anyOf(PluginManager::plugins(),
-                                                 Utils::equal(&PluginSpec::name,
-                                                              QString("ProjectExplorer")));
+    const bool hasProjectExplorer = PluginManager::specExists("projectexplorer");
     return hasProjectExplorer ? QStringList({"-lastsession"}) : QStringList();
 }
 
-// should be in sync with src/plugins/coreplugin/icore.cpp -> FilePath ICore::crashReportsPath()
-// and src\tools\qml2puppet\qml2puppet\qmlpuppet.cpp -> QString crashReportsPath()
-QString crashReportsPath()
-{
-    std::unique_ptr<QSettings> settings(createUserSettings());
-    QFileInfo(settings->fileName()).path() + "/crashpad_reports";
-    if (Utils::HostOsInfo::isMacHost())
-        return QFileInfo(createUserSettings()->fileName()).path() + "/crashpad_reports";
-    else
-        return QCoreApplication::applicationDirPath()
-                + '/' + RELATIVE_LIBEXEC_PATH + "crashpad_reports";
-}
-
 #ifdef ENABLE_CRASHPAD
-bool startCrashpad(const QString &libexecPath, bool crashReportingEnabled)
+void startCrashpad(const AppInfo &appInfo, bool crashReportingEnabled)
 {
+    if (!crashReportingEnabled)
+        return;
+
     using namespace crashpad;
 
     // Cache directory that will store crashpad information and minidumps
-    QString databasePath = QDir::cleanPath(crashReportsPath());
-    QString handlerPath = QDir::cleanPath(libexecPath + "/crashpad_handler");
+    const QString databasePath = appInfo.crashReports.path();
+    const QString handlerPath = (appInfo.libexec / "crashpad_handler").path();
 #ifdef Q_OS_WIN
-    handlerPath += ".exe";
     base::FilePath database(databasePath.toStdWString());
-    base::FilePath handler(handlerPath.toStdWString());
+    base::FilePath handler(HostOsInfo::withExecutableSuffix(handlerPath).toStdWString());
 #elif defined(Q_OS_MACOS) || defined(Q_OS_LINUX)
     base::FilePath database(databasePath.toStdString());
-    base::FilePath handler(handlerPath.toStdString());
+    base::FilePath handler(HostOsInfo::withExecutableSuffix(handlerPath).toStdString());
 #endif
 
     std::unique_ptr<CrashReportDatabase> db = CrashReportDatabase::Initialize(database);
@@ -452,12 +478,24 @@ bool startCrashpad(const QString &libexecPath, bool crashReportingEnabled)
     annotations["sha1"] = Core::Constants::IDE_REVISION_STR;
 #endif
 
+    CrashpadInfo::GetCrashpadInfo()->set_crashpad_handler_behavior(crashpad::TriState::kEnabled);
+    if (HostOsInfo::isWindowsHost()) {
+        // reduces the size of crash reports, which can be large on Windows
+        CrashpadInfo::GetCrashpadInfo()
+            ->set_gather_indirectly_referenced_memory(crashpad::TriState::kEnabled, 0);
+    }
+
+    // Explicitly enable Crashpad handling. This is the default in vanilla Crashpad,
+    // but we use a version that only handles processes that enable it explicitly,
+    // so we do not handle arbitrary subprocesses
+    CrashpadInfo::GetCrashpadInfo()->set_crashpad_handler_behavior(crashpad::TriState::kEnabled);
+
     // Optional arguments to pass to the handler
     std::vector<std::string> arguments;
     arguments.push_back("--no-rate-limit");
 
     CrashpadClient *client = new CrashpadClient();
-    bool success = client->StartHandler(
+    client->StartHandler(
         handler,
         database,
         database,
@@ -465,19 +503,123 @@ bool startCrashpad(const QString &libexecPath, bool crashReportingEnabled)
         annotations,
         arguments,
         /* restartable */ true,
-        /* asynchronous_start */ true
-    );
-
-    return success;
+        /* asynchronous_start */ true);
 }
 #endif
 
+#ifdef ENABLE_SENTRY
+void configureSentry(const AppInfo &appInfo, bool crashReportingEnabled)
+{
+    if (!crashReportingEnabled)
+        return;
+
+    sentry_options_t *options = sentry_options_new();
+    sentry_options_set_dsn(options, SENTRY_DSN);
+#ifdef Q_OS_WIN
+    sentry_options_set_database_pathw(options, appInfo.crashReports.nativePath().toStdWString().c_str());
+#else
+    sentry_options_set_database_path(options, appInfo.crashReports.nativePath().toUtf8().constData());
+#endif
+#ifdef SENTRY_CRASHPAD_PATH
+    if (const FilePath handlerpath = appInfo.libexec / "crashpad_handler"; handlerpath.exists()) {
+        sentry_options_set_handler_path(options, handlerpath.nativePath().toUtf8().constData());
+    } else if (const auto fallback = FilePath::fromString(SENTRY_CRASHPAD_PATH); fallback.exists()) {
+        sentry_options_set_handler_path(options, fallback.nativePath().toUtf8().constData());
+    } else {
+        qCWarning(sentryLog) << "Failed to find crashpad_handler for Sentry crash reports.";
+    }
+#endif
+    const QString release = QString(SENTRY_PROJECT) + "@" + QCoreApplication::applicationVersion();
+    sentry_options_set_release(options, release.toUtf8().constData());
+    sentry_options_set_debug(options, sentryLog().isDebugEnabled() ? 1 : 0);
+    sentry_init(options);
+}
+#endif
+
+class ShowInGuiHandler
+{
+public:
+    ShowInGuiHandler()
+    {
+        instance = this;
+        oldHandler = qInstallMessageHandler(log);
+    }
+    ~ShowInGuiHandler() { qInstallMessageHandler(oldHandler); };
+
+private:
+    static void log(QtMsgType type, const QMessageLogContext &context, const QString &msg)
+    {
+        instance->messages += msg;
+        if (type == QtFatalMsg) {
+            // Show some kind of GUI with collected messages before exiting.
+            // For Windows, Qt already uses a dialog.
+            if (HostOsInfo::isLinuxHost()) {
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 6, 0) && QT_VERSION < QT_VERSION_CHECK(6, 6, 1))
+                // Information about potentially missing libxcb-cursor0 is printed by Qt since Qt 6.5.3 and Qt 6.6.1
+                // Add it manually for other versions >= 6.5.0
+                instance->messages.prepend("From 6.5.0, xcb-cursor0 or libxcb-cursor0 is needed to "
+                                           "load the Qt xcb platform plugin.");
+#endif
+                if (QFile::exists("/usr/bin/xmessage"))
+                    QProcess::startDetached("/usr/bin/xmessage", {instance->messages.join("\n")});
+            } else if (HostOsInfo::isMacHost()) {
+                QProcess::startDetached("/usr/bin/osascript",
+                                        {"-e",
+                                         "display dialog \""
+                                             + instance->messages.join("\n").replace("\"", "\\\"")
+                                             + "\" buttons \"OK\" with title \""
+                                             + Core::Constants::IDE_DISPLAY_NAME
+                                             + " Failed to Start\""});
+            }
+        }
+        instance->oldHandler(type, context, msg);
+    };
+
+    static ShowInGuiHandler *instance;
+    QStringList messages;
+    QtMessageHandler oldHandler = nullptr;
+};
+
+ShowInGuiHandler *ShowInGuiHandler::instance = nullptr;
+
+FilePath userPluginsRoot()
+{
+    FilePath rootPath = FilePath::fromUserInput(
+        QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation));
+
+    if (HostOsInfo::isAnyUnixHost() && !HostOsInfo::isMacHost())
+        rootPath /= "data";
+
+    rootPath /= Core::Constants::IDE_SETTINGSVARIANT_STR;
+
+    rootPath /= QLatin1StringView(
+        HostOsInfo::isMacHost() ? Core::Constants::IDE_DISPLAY_NAME : Core::Constants::IDE_ID);
+    rootPath /= "plugins";
+
+    rootPath /= Core::Constants::IDE_VERSION_LONG;
+
+    return rootPath;
+}
+
+FilePath userResourcePath(const QString &settingsPath, const QString &appId)
+{
+    const FilePath configDir = FilePath::fromUserInput(settingsPath).parentDir();
+    const FilePath urp = configDir / appId;
+
+    if (!urp.exists()) {
+        if (!urp.createDir())
+            qWarning() << "could not create" << urp;
+    }
+
+    return urp;
+}
+
 int main(int argc, char **argv)
 {
     Restarter restarter(argc, argv);
-    Utils::Environment::systemEnvironment(); // cache system environment before we do any changes
+    Environment::systemEnvironment(); // cache system environment before we do any changes
 
-    Utils::FSEngine fileSystemEngine;
+    FSEngine fileSystemEngine;
 
     // Manually determine various command line options
     // We can't use the regular way of the plugin manager,
@@ -485,24 +627,42 @@ int main(int argc, char **argv)
     Options options = parseCommandLine(argc, argv);
     applicationDirPath(argv[0]);
 
-    const bool hasStyleOption = Utils::findOrDefault(options.appArguments, [](char *arg) {
-        return strcmp(arg, "-style") == 0;
-    });
-
-    if (qEnvironmentVariableIsSet("QTC_DO_NOT_PROPAGATE_LD_PRELOAD")) {
-        Utils::Environment::modifySystemEnvironment(
-            {{"LD_PRELOAD", "", Utils::EnvironmentItem::Unset}});
+    // Remove entries from environment variables that were set up by Qt Creator to run
+    // the application (in this case, us).
+    // TODO: We should be able to merge at least some of the stuff below with similar intent
+    //       into a more generalized version of this.
+    EnvironmentItems specialItems;
+    EnvironmentItems diff;
+    Environment::systemEnvironment().forEachEntry(
+        [&specialItems](const QString &name, const QString &value, bool enabled) {
+            if (enabled && name.startsWith("_QTC_"))
+                specialItems.emplaceBack(name, value, EnvironmentItem::SetEnabled);
+        });
+    for (const EnvironmentItem &item : std::as_const(specialItems)) {
+        const QString varName = item.name.mid(5);
+        const FilePaths addedPaths
+            = Environment::pathListFromValue(item.value, HostOsInfo::hostOs());
+        FilePaths allPaths = Environment::systemEnvironment().pathListValue(varName);
+        Utils::erase(allPaths, [&addedPaths](const FilePath &p) {
+            return addedPaths.contains(p);
+        });
+        diff.emplaceBack(
+            varName,
+            Environment::valueFromPathList(allPaths, HostOsInfo::hostOs()),
+            EnvironmentItem::SetEnabled);
+        diff.emplaceBack(item.name, "", EnvironmentItem::Unset);
     }
+    Environment::modifySystemEnvironment(diff);
+
+    if (qEnvironmentVariableIsSet("QTC_DO_NOT_PROPAGATE_LD_PRELOAD"))
+        Environment::modifySystemEnvironment({{"LD_PRELOAD", "", EnvironmentItem::Unset}});
 
     auto restoreEnvVarFromSquish = [](const QByteArray &squishVar, const QString &var) {
         if (qEnvironmentVariableIsSet(squishVar)) {
-            Utils::Environment::modifySystemEnvironment(
-                {{var, "", Utils::EnvironmentItem::Unset}});
+            Environment::modifySystemEnvironment({{var, "", EnvironmentItem::Unset}});
             const QString content = qEnvironmentVariable(squishVar);
-            if (!content.isEmpty()) {
-                Utils::Environment::modifySystemEnvironment(
-                    {{var, content, Utils::EnvironmentItem::Prepend}});
-            }
+            if (!content.isEmpty())
+                Environment::modifySystemEnvironment({{var, content, EnvironmentItem::Prepend}});
         }
     };
 
@@ -511,30 +671,30 @@ int main(int argc, char **argv)
 
     if (options.userLibraryPath) {
         if ((*options.userLibraryPath).isEmpty()) {
-            Utils::Environment::modifySystemEnvironment(
-                {{"LD_LIBRARY_PATH", "", Utils::EnvironmentItem::Unset}});
+            Environment::modifySystemEnvironment(
+                {{"LD_LIBRARY_PATH", "", EnvironmentItem::Unset}});
         } else {
-            Utils::Environment::modifySystemEnvironment(
-                {{"LD_LIBRARY_PATH", *options.userLibraryPath, Utils::EnvironmentItem::SetEnabled}});
+            Environment::modifySystemEnvironment(
+                {{"LD_LIBRARY_PATH", *options.userLibraryPath, EnvironmentItem::SetEnabled}});
         }
     }
 
-    qputenv("QSG_RHI_BACKEND", "opengl");
-    QGuiApplication::setHighDpiScaleFactorRoundingPolicy(
-                Qt::HighDpiScaleFactorRoundingPolicy::Round);
+    if (HostOsInfo::isMacHost()) {
+        QSurfaceFormat surfaceFormat;
+        surfaceFormat.setStencilBufferSize(8);
+        surfaceFormat.setDepthBufferSize(24);
+        surfaceFormat.setVersion(4, 1);
+        surfaceFormat.setProfile(QSurfaceFormat::CoreProfile);
+        QSurfaceFormat::setDefaultFormat(surfaceFormat);
+    }
 
     if (qEnvironmentVariableIsSet("QTCREATOR_DISABLE_NATIVE_MENUBAR")
             || qgetenv("XDG_CURRENT_DESKTOP").startsWith("Unity")) {
         QApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
     }
 
-    if (Utils::HostOsInfo::isLinuxHost() && !qEnvironmentVariableIsSet("GTK_THEME"))
-        // Work around QTCREATORBUG-28497:
-        // Prevent Qt's GTK3 platform theme plugin from enforcing a dark palette
-        qputenv("GTK_THEME", ":light");
-
 #if defined(QTC_FORCE_XCB)
-    if (Utils::HostOsInfo::isLinuxHost() && !qEnvironmentVariableIsSet("QT_QPA_PLATFORM")) {
+    if (HostOsInfo::isLinuxHost() && !qEnvironmentVariableIsSet("QT_QPA_PLATFORM")) {
         // Enforce XCB on Linux/Gnome, if the user didn't override via QT_QPA_PLATFORM
         // This was previously done in Qt, but removed in Qt 6.3. We found that bad things can still happen,
         // like the Wayland session simply crashing when starting Qt Creator.
@@ -553,7 +713,7 @@ int main(int argc, char **argv)
     }
 #endif
 
-    Utils::TemporaryDirectory::setMasterTemporaryDirectory(QDir::tempPath() + "/" + Core::Constants::IDE_CASED_ID + "-XXXXXX");
+    TemporaryDirectory::setMasterTemporaryDirectory(QDir::tempPath() + "/" + Core::Constants::IDE_CASED_ID + "-XXXXXX");
 
 #ifdef Q_OS_MACOS
     // increase the number of file that can be opened in Qt Creator.
@@ -564,9 +724,9 @@ int main(int argc, char **argv)
     setrlimit(RLIMIT_NOFILE, &rl);
 #endif
 
-    QScopedPointer<Utils::TemporaryDirectory> temporaryCleanSettingsDir;
+    QScopedPointer<TemporaryDirectory> temporaryCleanSettingsDir;
     if (options.settingsPath.isEmpty() && (options.hasTestOption || options.wantsCleanSettings)) {
-        temporaryCleanSettingsDir.reset(new Utils::TemporaryDirectory("qtc-test-settings"));
+        temporaryCleanSettingsDir.reset(new TemporaryDirectory("qtc-test-settings"));
         if (!temporaryCleanSettingsDir->isValid())
             return 1;
         options.settingsPath = temporaryCleanSettingsDir->path().path();
@@ -581,22 +741,29 @@ int main(int argc, char **argv)
     // Since we do not have a QApplication yet, we cannot rely on QApplication::applicationDirPath()
     // though. So we set up install settings with a educated guess here, and re-setup it later.
     setupInstallSettings(options.installSettingsPath);
+    setupAccessibility();
     setHighDpiEnvironmentVariable();
+    setRHIOpenGLVariable();
 
     SharedTools::QtSingleApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
 
     int numberOfArguments = static_cast<int>(options.appArguments.size());
 
-    std::unique_ptr<SharedTools::QtSingleApplication>
-        appPtr(SharedTools::createApplication(QLatin1String(Core::Constants::IDE_DISPLAY_NAME),
-                                              numberOfArguments, options.appArguments.data()));
+    // create a custom Qt message handler that shows messages in a bare bones UI
+    // if creation of the QGuiApplication fails.
+    auto handler = std::make_unique<ShowInGuiHandler>();
+    const QString singleAppId = QString(Core::Constants::IDE_DISPLAY_NAME)
+                                + options.singleAppIdPostfix;
+    std::unique_ptr<SharedTools::QtSingleApplication> appPtr(
+        SharedTools::createApplication(singleAppId, numberOfArguments, options.appArguments.data()));
+    handler.reset();
     SharedTools::QtSingleApplication &app = *appPtr;
     QCoreApplication::setApplicationName(Core::Constants::IDE_CASED_ID);
     QCoreApplication::setApplicationVersion(QLatin1String(Core::Constants::IDE_VERSION_LONG));
     QCoreApplication::setOrganizationName(QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR));
     QGuiApplication::setApplicationDisplayName(Core::Constants::IDE_DISPLAY_NAME);
 
-    const QScopeGuard cleanup([] { Utils::Singleton::deleteAll(); });
+    const QScopeGuard cleanup([] { ProcessReaper::deleteAll(); });
 
     const QStringList pluginArguments = app.arguments();
 
@@ -606,48 +773,91 @@ int main(int argc, char **argv)
     const QStringList installPluginPaths = getInstallPluginPaths();
     // Re-setup install settings for real
     setupInstallSettings(options.installSettingsPath);
-    Utils::QtcSettings *settings = createUserSettings();
-    Utils::QtcSettings *globalSettings
-        = new Utils::QtcSettings(QSettings::IniFormat,
-                                 QSettings::SystemScope,
-                                 QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
-                                 QLatin1String(Core::Constants::IDE_CASED_ID));
-    Utils::TerminalCommand::setSettings(settings);
+    QtcSettings *settings = createUserSettings();
+    QtcSettings *installSettings
+        = new QtcSettings(QSettings::IniFormat,
+                          QSettings::SystemScope,
+                          QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
+                          QLatin1String(Core::Constants::IDE_CASED_ID));
+    // warn if -installsettings points to a place where no install settings are located
+    if (!options.installSettingsPath.isEmpty() && !QFileInfo::exists(installSettings->fileName())) {
+        displayError(QLatin1String("The install settings \"%1\" do not exist. The %2 option must "
+                                   "point to a path with existing settings, excluding the %3 part "
+                                   "of the path.")
+                         .arg(QDir::toNativeSeparators(installSettings->fileName()),
+                              INSTALL_SETTINGS_OPTION,
+                              Core::Constants::IDE_SETTINGSVARIANT_STR));
+    }
+    TerminalCommand::setSettings(settings);
     setPixmapCacheLimit();
     loadFonts();
 
-    if (Utils::HostOsInfo::isWindowsHost() && !qFuzzyCompare(qApp->devicePixelRatio(), 1.0)
-        && !hasStyleOption) {
-        QApplication::setStyle(QLatin1String("fusion"));
+    if (Utils::HostOsInfo::isWindowsHost() && !options.hasStyleOption) {
+        // The Windows 11 default style (Qt 6.7) has major issues, therefore
+        // set the previous default style: "windowsvista"
+        // FIXME: check newer Qt Versions
+        QApplication::setStyle(QLatin1String("windowsvista"));
+
+        // On scaling different than 100% or 200% use the "fusion" style
+        qreal tmp;
+        const bool fractionalDpi = !qFuzzyIsNull(std::modf(qApp->devicePixelRatio(), &tmp));
+        if (fractionalDpi)
+            QApplication::setStyle(QLatin1String("fusion"));
     }
+
     const int threadCount = QThreadPool::globalInstance()->maxThreadCount();
     QThreadPool::globalInstance()->setMaxThreadCount(qMax(4, 2 * threadCount));
 
-    const QString libexecPath = QCoreApplication::applicationDirPath()
-            + '/' + RELATIVE_LIBEXEC_PATH;
-#ifdef ENABLE_QT_BREAKPAD
-    QtSystemExceptionHandler systemExceptionHandler(libexecPath);
-#else
-    // Display a backtrace once a serious signal is delivered (Linux only).
-    CrashHandlerSetup setupCrashHandler(Core::Constants::IDE_DISPLAY_NAME,
-                                        CrashHandlerSetup::EnableRestart, libexecPath);
-#endif
+    using namespace Core;
+    const FilePath appDirPath = FilePath::fromUserInput(QApplication::applicationDirPath());
+    AppInfo info;
+    info.author = Constants::IDE_AUTHOR;
+    info.copyright = Constants::IDE_COPYRIGHT;
+    info.displayVersion = Constants::IDE_VERSION_DISPLAY;
+    info.id = Constants::IDE_ID;
+    info.revision = Constants::IDE_REVISION_STR;
+    info.revisionUrl = Constants::IDE_REVISION_URL;
+    info.userFileExtension = Constants::IDE_PROJECT_USER_FILE_EXTENSION;
+    info.plugins = (appDirPath / RELATIVE_PLUGIN_PATH).cleanPath();
+    info.userPluginsRoot = userPluginsRoot();
+    info.resources = (appDirPath / RELATIVE_DATA_PATH).cleanPath();
+    info.userResources = userResourcePath(settings->fileName(), Constants::IDE_ID);
+    info.libexec = (appDirPath / RELATIVE_LIBEXEC_PATH).cleanPath();
+    // sync with src\tools\qmlpuppet\qmlpuppet\qmlpuppet.cpp -> QString crashReportsPath()
+    info.crashReports = info.userResources / "crashpad_reports";
+    info.luaPlugins = info.resources / "lua-plugins";
+    info.userLuaPlugins = info.userResources / "lua-plugins";
+    Utils::Internal::setAppInfo(info);
 
-#ifdef ENABLE_CRASHPAD
-    bool crashReportingEnabled = settings->value("CrashReportingEnabled", false).toBool();
-    startCrashpad(libexecPath, crashReportingEnabled);
+    // Display a backtrace once a serious signal is delivered (Linux only).
+    CrashHandlerSetup setupCrashHandler(
+        Core::Constants::IDE_DISPLAY_NAME, CrashHandlerSetup::EnableRestart, info.libexec.path());
+
+    // depends on AppInfo and QApplication being created
+    const bool crashReportingEnabled = settings->value("CrashReportingEnabled", false).toBool();
+
+#if defined(ENABLE_CRASHPAD)
+    startCrashpad(info, crashReportingEnabled);
+#elif defined(ENABLE_SENTRY)
+    configureSentry(info, crashReportingEnabled);
+#else
+    Q_UNUSED(crashReportingEnabled)
 #endif
 
     PluginManager pluginManager;
     PluginManager::setPluginIID(QLatin1String("org.qt-project.Qt.QtCreatorPlugin"));
-    PluginManager::setGlobalSettings(globalSettings);
+    PluginManager::setInstallSettings(installSettings);
     PluginManager::setSettings(settings);
     PluginManager::startProfiling();
 
+    BaseAspect::setQtcSettings(settings);
+
     QTranslator translator;
     QTranslator qtTranslator;
     QStringList uiLanguages = QLocale::system().uiLanguages();
-    QString overrideLanguage = settings->value(QLatin1String("General/OverrideLanguage")).toString();
+    const QString overrideLanguage = options.hasTestOption
+                                         ? QString("C") // force built-in when running tests
+                                         : settings->value("General/OverrideLanguage").toString();
     if (!overrideLanguage.isEmpty())
         uiLanguages.prepend(overrideLanguage);
     if (!options.uiLanguage.isEmpty())
@@ -666,11 +876,13 @@ int main(int argc, char **argv)
                 break;
             }
             Q_UNUSED(translator.load(QString())); // unload()
-        } else if (locale == QLatin1String("C") /* overrideLanguage == "English" */) {
-            // use built-in
-            break;
-        } else if (locale.startsWith(QLatin1String("en")) /* "English" is built-in */) {
-            // use built-in
+        } else if (
+            locale == QLatin1String("C") /* overrideLanguage == "English" */
+            || locale.startsWith(QLatin1String("en")) /* "English" is built-in */) {
+            // Load any spelling fixes that might have been done temporarily after string freeze,
+            // for the rest use the built-in source text
+            if (translator.load("qtcreator_en", creatorTrPath))
+                app.installTranslator(&translator);
             break;
         }
     }
@@ -679,15 +891,21 @@ int main(int argc, char **argv)
     if (!overrideCodecForLocale.isEmpty())
         QTextCodec::setCodecForLocale(QTextCodec::codecForName(overrideCodecForLocale));
 
-    app.setDesktopFileName("org.qt-project.qtcreator.desktop");
+    app.setDesktopFileName(IDE_APP_ID);
 
     // Make sure we honor the system's proxy settings
     QNetworkProxyFactory::setUseSystemConfiguration(true);
 
+    PluginManager::removePluginsAfterRestart();
+
+    // We need to install plugins before we scan for them.
+    PluginManager::installPluginsAfterRestart();
+
     // Load
-    const QStringList pluginPaths = getPluginPaths() + installPluginPaths
-                                    + options.customPluginPaths;
-    PluginManager::setPluginPaths(pluginPaths);
+    const QStringList pluginPaths = installPluginPaths + options.customPluginPaths;
+    PluginManager::setPluginPaths(
+        getPluginPaths() + Utils::transform(pluginPaths, &FilePath::fromUserInput));
+
     QMap<QString, QString> foundAppOptions;
     if (pluginArguments.size() > 1) {
         QMap<QString, bool> appOptions;
@@ -700,9 +918,8 @@ int main(int argc, char **argv)
         appOptions.insert(QLatin1String(CLIENT_OPTION), false);
         appOptions.insert(QLatin1String(PID_OPTION), true);
         appOptions.insert(QLatin1String(BLOCK_OPTION), false);
-        QString errorMessage;
-        if (!PluginManager::parseOptions(pluginArguments, appOptions, &foundAppOptions, &errorMessage)) {
-            displayError(errorMessage);
+        if (Result<> res = PluginManager::parseOptions(pluginArguments, appOptions, &foundAppOptions); !res) {
+            displayError(res.error());
             printHelp(QFileInfo(app.applicationFilePath()).baseName());
             return -1;
         }
@@ -717,14 +934,7 @@ int main(int argc, char **argv)
             settingspath};
     PluginManager::setCreatorProcessData(processData);
 
-    const PluginSpecSet plugins = PluginManager::plugins();
-    PluginSpec *coreplugin = nullptr;
-    for (PluginSpec *spec : plugins) {
-        if (spec->name() == QLatin1String(corePluginNameC)) {
-            coreplugin = spec;
-            break;
-        }
-    }
+    PluginSpec *coreplugin = PluginManager::specById(QLatin1String(corePluginIdC));
     if (!coreplugin) {
         QString nativePaths = QDir::toNativeSeparators(pluginPaths.join(QLatin1Char(',')));
         const QString reason = QCoreApplication::translate("Application", "Could not find Core plugin in %1").arg(nativePaths);
@@ -803,5 +1013,17 @@ int main(int argc, char **argv)
     // shutdown plugin manager on the exit
     QObject::connect(&app, &QCoreApplication::aboutToQuit, &pluginManager, &PluginManager::shutdown);
 
-    return restarter.restartOrExit(app.exec());
+    if (Utils::HostOsInfo::isWindowsHost()) {
+        // Workaround for QTBUG-130696 and QTCREATORBUG-31890
+        QApplication::setEffectEnabled(Qt::UI_FadeMenu, false);
+
+        // Disable menu animation which just looks bad
+        QApplication::setEffectEnabled(Qt::UI_AnimateMenu, false);
+    }
+
+    const int exitCode = restarter.restartOrExit(app.exec());
+#ifdef ENABLE_SENTRY
+    sentry_close();
+#endif
+    return exitCode;
 }
