@@ -3,6 +3,7 @@
 
 #include "cmakeproject.h"
 
+#include "cmakebuildsystem.h"
 #include "cmakekitaspect.h"
 #include "cmakeprojectconstants.h"
 #include "cmakeprojectimporter.h"
@@ -52,8 +53,12 @@ CMakeProject::CMakeProject(const FilePath &fileName)
 {
     setId(CMakeProjectManager::Constants::CMAKE_PROJECT_ID);
     setProjectLanguages(Core::Context(ProjectExplorer::Constants::CXX_LANGUAGE_ID));
-    setDisplayName(projectDirectory().fileName());
+    setDisplayName(projectDisplayName(projectFilePath()));
     setCanBuildProducts();
+    setBuildSystemCreator<CMakeBuildSystem>("cmake");
+
+    // Allow presets to check if being run under Qt Creator
+    Environment::modifySystemEnvironment({{"QTC_RUN", "1"}});
 
     // This only influences whether 'Install into temporary host directory'
     // will show up by default enabled in some remote deploy configurations.
@@ -74,18 +79,10 @@ CMakeProject::~CMakeProject()
 Tasks CMakeProject::projectIssues(const Kit *k) const
 {
     Tasks result = Project::projectIssues(k);
-
-    if (!CMakeKitAspect::cmakeTool(k))
-        result.append(createProjectTask(Task::TaskType::Error, Tr::tr("No cmake tool set.")));
-    if (ToolchainKitAspect::toolChains(k).isEmpty())
-        result.append(createProjectTask(Task::TaskType::Warning, Tr::tr("No compilers set in kit.")));
-
     result.append(m_issues);
-
     return result;
 }
 
-
 ProjectImporter *CMakeProject::projectImporter() const
 {
     if (!m_projectImporter)
@@ -95,7 +92,7 @@ ProjectImporter *CMakeProject::projectImporter() const
 
 void CMakeProject::addIssue(IssueType type, const QString &text)
 {
-    m_issues.append(createProjectTask(type, text));
+    m_issues.append(createTask(type, text));
 }
 
 void CMakeProject::clearIssues()
@@ -258,6 +255,44 @@ void CMakeProject::setupBuildPresets(Internal::PresetsData &presetsData)
     }
 }
 
+QString CMakeProject::projectDisplayName(const Utils::FilePath &projectFilePath)
+{
+    const QString fallbackDisplayName = projectFilePath.absolutePath().fileName();
+
+    Result<QByteArray> fileContent = projectFilePath.fileContents();
+    cmListFile cmakeListFile;
+    std::string errorString;
+    if (fileContent) {
+        fileContent = fileContent->replace("\r\n", "\n");
+        if (!cmakeListFile.ParseString(
+                fileContent->toStdString(), projectFilePath.fileName().toStdString(), errorString)) {
+            return fallbackDisplayName;
+        }
+    }
+
+    QHash<QString, QString> setVariables;
+    for (const auto &func : cmakeListFile.Functions) {
+        if (func.LowerCaseName() == "set" && func.Arguments().size() == 2)
+            setVariables.insert(
+                QString::fromUtf8(func.Arguments()[0].Value),
+                QString::fromUtf8(func.Arguments()[1].Value));
+
+        if (func.LowerCaseName() == "project" && func.Arguments().size() > 0) {
+            const QString projectName = QString::fromUtf8(func.Arguments()[0].Value);
+            if (projectName.startsWith("${") && projectName.endsWith("}")) {
+                const QString projectVar = projectName.mid(2, projectName.size() - 3);
+                if (setVariables.contains(projectVar))
+                    return setVariables.value(projectVar);
+                else
+                    return fallbackDisplayName;
+            }
+            return projectName;
+        }
+    }
+
+    return fallbackDisplayName;
+}
+
 Internal::CMakeSpecificSettings &CMakeProject::settings()
 {
     return m_settings;
@@ -336,7 +371,7 @@ void CMakeProject::readPresets()
     m_presetsData = combinePresets(cmakePresetsData, cmakeUserPresetsData);
     setupBuildPresets(m_presetsData);
 
-    for (const auto &configPreset : m_presetsData.configurePresets) {
+    for (const auto &configPreset : std::as_const(m_presetsData.configurePresets)) {
         if (configPreset.hidden)
             continue;
 
@@ -354,15 +389,6 @@ FilePath CMakeProject::buildDirectoryToImport() const
     return m_buildDirToImport;
 }
 
-bool CMakeProject::setupTarget(Target *t)
-{
-    t->updateDefaultBuildConfigurations();
-    if (t->buildConfigurations().isEmpty())
-        return false;
-    t->updateDefaultDeployConfigurations();
-    return true;
-}
-
 ProjectExplorer::DeploymentKnowledge CMakeProject::deploymentKnowledge() const
 {
     return !files([](const ProjectExplorer::Node *n) {
