@@ -102,12 +102,18 @@ GitSettings::GitSettings()
     instantBlameIgnoreLineMoves.setLabelText(trIgnoreLineMoves());
     instantBlameIgnoreLineMoves.setToolTip(
         Tr::tr("Finds the commit that introduced the line before it was moved."));
+    instantBlameShowSubject.setSettingsKey("GitInstantShowSubject");
+    instantBlameShowSubject.setDefaultValue(false);
+    instantBlameShowSubject.setLabelText(Tr::tr("Show commit subject"));
+    instantBlameShowSubject.setToolTip(
+        Tr::tr("Adds the commit subject directly to the annotation."));
 
     graphLog.setSettingsKey("GraphLog");
 
     colorLog.setSettingsKey("ColorLog");
     colorLog.setDefaultValue(true);
 
+    allBranches.setSettingsKey("AllBranches");
     firstParent.setSettingsKey("FirstParent");
 
     followRenames.setSettingsKey("FollowRenames");
@@ -151,8 +157,13 @@ GitSettings::GitSettings()
 
             Group {
                 title(Tr::tr("Instant Blame")),
-                instantBlame.groupChecker(),
-                Row { instantBlameIgnoreSpaceChanges, instantBlameIgnoreLineMoves, st },
+                groupChecker(instantBlame.groupChecker()),
+                Row {
+                    instantBlameIgnoreSpaceChanges,
+                    instantBlameIgnoreLineMoves,
+                    instantBlameShowSubject,
+                    st
+                },
             },
 
             st
@@ -164,7 +175,7 @@ GitSettings::GitSettings()
     readSettings();
 }
 
-expected_str<FilePath> GitSettings::gitExecutable() const
+Result<FilePath> GitSettings::gitExecutable() const
 {
     if (tryResolve) {
         resolvedBinPath = binaryPath();
