@@ -196,6 +196,7 @@ const PackageDescription
                             {},
                             VersionDetection{},
                             false,
+                            false,
                             Utils::PathChooser::Kind::ExistingDirectory};
 
 const McuTargetDescription::Platform platformDescription{id,
@@ -731,7 +732,7 @@ void McuSupportTest::test_legacy_createPackagesWithCorrespondingSettings()
     const McuTargetDescription description = parseDescriptionJson(json.toLocal8Bit());
     auto [targets, packages]{
         targetsFromDescriptions({description}, settingsMockPtr, sdkPackagePtr, runLegacy)};
-    Q_UNUSED(targets);
+    Q_UNUSED(targets)
 
     QSet<Key> settings = transform<QSet<Key>>(packages, [](const auto &package) {
         return package->settingsKey();
@@ -851,6 +852,7 @@ void McuSupportTest::test_useFallbackPathForToolchainWhenPathFromSettingsIsNotAv
                                            {},
                                            VersionDetection{},
                                            false,
+                                           false,
                                            Utils::PathChooser::Kind::ExistingDirectory};
     McuTargetDescription::Toolchain toolchainDescription{armGcc, {}, compilerDescription, {}};
 
@@ -875,6 +877,7 @@ void McuSupportTest::test_usePathFromSettingsForToolchainPath()
                                            {},
                                            VersionDetection{},
                                            false,
+                                           false,
                                            Utils::PathChooser::Kind::ExistingDirectory};
     McuTargetDescription::Toolchain toolchainDescription{armGcc, {}, compilerDescription, {}};
 
@@ -1008,7 +1011,7 @@ void McuSupportTest::test_legacy_createTargetWithToolchainPackages()
 
     const auto [targets, packages]{
         targetsFromDescriptions({description}, settingsMockPtr, sdkPackagePtr, runLegacy)};
-    Q_UNUSED(packages);
+    Q_UNUSED(packages)
 
     verifyTargetToolchains(targets,
                            toolchainFilePath,
@@ -1042,7 +1045,7 @@ void McuSupportTest::test_createTargetWithToolchainPackages()
     const McuTargetDescription description = parseDescriptionJson(json.toLocal8Bit());
     const auto [targets, packages]{
         targetsFromDescriptions({description}, settingsMockPtr, sdkPackagePtr, !runLegacy)};
-    Q_UNUSED(packages);
+    Q_UNUSED(packages)
 
     const auto qtForMCUsSDK = findOrDefault(packages, [](const McuPackagePtr &pkg) {
         return (pkg->cmakeVariableName() == QUL_CMAKE_VAR);
