@@ -232,7 +232,7 @@ TaskWindow::TaskWindow() : d(std::make_unique<TaskWindowPrivate>())
     connect(hub, &TaskHub::taskFileNameUpdated, this, &TaskWindow::updatedTaskFileName);
     connect(hub, &TaskHub::tasksCleared, this, &TaskWindow::clearTasks);
     connect(hub, &TaskHub::categoryVisibilityChanged, this, &TaskWindow::setCategoryVisibility);
-    connect(hub, &TaskHub::popupRequested, this, &TaskWindow::popup);
+    connect(hub, &TaskHub::popupRequested, this, &TaskWindow::popup, Qt::QueuedConnection);
     connect(hub, &TaskHub::showTask, this, &TaskWindow::showTask);
     connect(hub, &TaskHub::openTask, this, &TaskWindow::openTask);
 
@@ -535,46 +535,46 @@ bool TaskWindow::canPrevious() const
 
 void TaskWindow::goToNext()
 {
-    if (!canNext())
-        return;
-    QModelIndex startIndex = d->m_treeView.currentIndex();
-    QModelIndex currentIndex = startIndex;
-
-    if (startIndex.isValid()) {
-        do {
-            int row = currentIndex.row() + 1;
-            if (row == d->m_filter->rowCount())
-                row = 0;
-            currentIndex = d->m_filter->index(row, 0);
-            if (d->m_filter->hasFile(currentIndex))
-                break;
-        } while (startIndex != currentIndex);
-    } else {
-        currentIndex = d->m_filter->index(0, 0);
-    }
-    d->m_treeView.setCurrentIndex(currentIndex);
-    triggerDefaultHandler(currentIndex);
+    if (canNext())
+        goToNextOrPrev(1);
 }
 
 void TaskWindow::goToPrev()
 {
-    if (!canPrevious())
-        return;
+    if (canPrevious())
+        goToNextOrPrev(-1);
+}
+
+void TaskWindow::goToNextOrPrev(int offset)
+{
     QModelIndex startIndex = d->m_treeView.currentIndex();
     QModelIndex currentIndex = startIndex;
+    QModelIndex actualNeighbor;
 
     if (startIndex.isValid()) {
         do {
-            int row = currentIndex.row() - 1;
-            if (row < 0)
+            int row = currentIndex.row() + offset;
+            if (row == d->m_filter->rowCount())
+                row = 0;
+            else if (row < 0)
                 row = d->m_filter->rowCount() - 1;
             currentIndex = d->m_filter->index(row, 0);
+            if (!actualNeighbor.isValid())
+                actualNeighbor = currentIndex;
             if (d->m_filter->hasFile(currentIndex))
                 break;
         } while (startIndex != currentIndex);
     } else {
         currentIndex = d->m_filter->index(0, 0);
     }
+
+    // We only consider elements with files, except if there are none at all, in which case
+    // we don't skip anything.
+    if (currentIndex == startIndex && actualNeighbor.isValid()
+        && !d->m_filter->hasFile(currentIndex)) {
+        currentIndex = actualNeighbor;
+    }
+
     d->m_treeView.setCurrentIndex(currentIndex);
     triggerDefaultHandler(currentIndex);
 }
