@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmljseditor.h"
 
@@ -30,10 +8,13 @@
 #include "qmljseditorconstants.h"
 #include "qmljseditordocument.h"
 #include "qmljseditorplugin.h"
+#include "qmljseditorsettings.h"
+#include "qmljseditortr.h"
 #include "qmljsfindreferences.h"
 #include "qmljshighlighter.h"
 #include "qmljshoverhandler.h"
 #include "qmljsquickfixassist.h"
+#include "qmllsclientsettings.h"
 #include "qmloutlinemodel.h"
 #include "quicktoolbar.h"
 
@@ -44,16 +25,21 @@
 
 #include <qmljstools/qmljsindenter.h>
 #include <qmljstools/qmljstoolsconstants.h>
+
+#include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectmanager.h>
+#include <projectexplorer/projectnodes.h>
+#include <projectexplorer/projecttree.h>
 
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/coreconstants.h>
+#include <coreplugin/coreplugintr.h>
 #include <coreplugin/designmode.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/id.h>
 #include <coreplugin/modemanager.h>
 
 #include <extensionsystem/pluginmanager.h>
@@ -67,17 +53,23 @@
 #include <texteditor/refactoroverlay.h>
 #include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/genericproposalmodel.h>
-#include <texteditor/texteditoractionhandler.h>
+#include <texteditor/colorpreviewhoverhandler.h>
+#include <texteditor/snippets/snippetprovider.h>
 #include <texteditor/textmark.h>
 
-#include <utils/delegates.h>
+#include <utils/algorithm.h>
 #include <utils/changeset.h>
+#include <utils/delegates.h>
+#include <utils/mimeconstants.h>
 #include <utils/qtcassert.h>
 #include <utils/uncommentselection.h>
 
+#include <languageclient/languageclientmanager.h>
+#include <languageclient/locatorfilter.h>
+#include <languageclient/languageclientsymbolsupport.h>
+
 #include <QComboBox>
 #include <QCoreApplication>
-#include <QFileInfo>
 #include <QHeaderView>
 #include <QMenu>
 #include <QMetaMethod>
@@ -86,20 +78,41 @@
 #include <QTextCodec>
 #include <QTimer>
 #include <QTreeView>
+#include <QDebug>
 
 enum {
     UPDATE_USES_DEFAULT_INTERVAL = 150,
     UPDATE_OUTLINE_INTERVAL = 500 // msecs after new semantic info has been arrived / cursor has moved
 };
 
+const char QML_JS_EDITOR_PLUGIN[] = "QmlJSEditorPlugin";
+const char QT_QUICK_TOOLBAR_MARKER_ID[] = "QtQuickToolbarMarkerId";
+
 using namespace Core;
 using namespace QmlJS;
 using namespace QmlJS::AST;
+using namespace QmlJSEditor::Internal;
 using namespace QmlJSTools;
 using namespace TextEditor;
+using namespace Utils;
 
 namespace QmlJSEditor {
-namespace Internal {
+
+enum QmllsFunctionality {
+    DefaultFunctionality,
+    ExperimentalFunctionality,
+};
+
+static LanguageClient::Client *getQmllsClient(
+    ProjectExplorer::Project *project, const FilePath &fileName, QmllsFunctionality functionality)
+{
+    if (functionality == ExperimentalFunctionality
+        && qmllsSettings()->useQmllsWithBuiltinCodemodelOnProject(project, fileName))
+        return nullptr;
+
+    auto client = LanguageClient::LanguageClientManager::clientForFilePath(fileName);
+    return client;
+}
 
 //
 // QmlJSEditorWidget
@@ -109,6 +122,9 @@ QmlJSEditorWidget::QmlJSEditorWidget()
 {
     m_findReferences = new FindReferences(this);
     setLanguageSettingsId(QmlJSTools::Constants::QML_JS_SETTINGS_ID);
+
+    connect(this, &QmlJSEditorWidget::toolbarOutlineChanged,
+            this, &QmlJSEditorWidget::updateOutline);
 }
 
 void QmlJSEditorWidget::finalizeInitialization()
@@ -118,7 +134,7 @@ void QmlJSEditorWidget::finalizeInitialization()
     m_updateUsesTimer.setInterval(UPDATE_USES_DEFAULT_INTERVAL);
     m_updateUsesTimer.setSingleShot(true);
     connect(&m_updateUsesTimer, &QTimer::timeout, this, &QmlJSEditorWidget::updateUses);
-    connect(this, &QPlainTextEdit::cursorPositionChanged,
+    connect(this, &PlainTextEdit::cursorPositionChanged,
             &m_updateUsesTimer, QOverload<>::of(&QTimer::start));
 
     m_updateOutlineIndexTimer.setInterval(UPDATE_OUTLINE_INTERVAL);
@@ -126,10 +142,8 @@ void QmlJSEditorWidget::finalizeInitialization()
     connect(&m_updateOutlineIndexTimer, &QTimer::timeout,
             this, &QmlJSEditorWidget::updateOutlineIndexNow);
 
-    textDocument()->setCodec(QTextCodec::codecForName("UTF-8")); // qml files are defined to be utf-8
-
     m_modelManager = ModelManagerInterface::instance();
-    m_contextPane = QmlJSEditorPlugin::quickToolBar();
+    m_contextPane = QuickToolBar::instance();
 
     m_modelManager->activateScan();
 
@@ -139,11 +153,11 @@ void QmlJSEditorWidget::finalizeInitialization()
     if (m_contextPane) {
         connect(this, &QmlJSEditorWidget::cursorPositionChanged,
                 &m_contextPaneTimer, QOverload<>::of(&QTimer::start));
-        connect(m_contextPane, &IContextPane::closed, this, &QmlJSEditorWidget::showTextMarker);
+        connect(m_contextPane, &QuickToolBar::closed, this, &QmlJSEditorWidget::showTextMarker);
     }
 
     connect(this->document(), &QTextDocument::modificationChanged,
-            this, &QmlJSEditorWidget::modificationChanged);
+            this, &QmlJSEditorWidget::updateModificationChange);
 
     connect(m_qmlJsEditorDocument, &QmlJSEditorDocument::updateCodeWarnings,
             this, &QmlJSEditorWidget::updateCodeWarnings);
@@ -155,6 +169,22 @@ void QmlJSEditorWidget::finalizeInitialization()
     createToolBar();
 }
 
+void QmlJSEditorWidget::restoreState(const QByteArray &state)
+{
+    using namespace Utils::Constants;
+    QStringList qmlTypes = {QML_MIMETYPE, QBS_MIMETYPE, QMLTYPES_MIMETYPE, QMLUI_MIMETYPE};
+
+    if (settings().foldAuxData() && qmlTypes.contains(textDocument()->mimeType())) {
+        int version = 0;
+        QDataStream stream(state);
+        stream >> version;
+        if (version < 1)
+            foldAuxiliaryData();
+    }
+
+    TextEditorWidget::restoreState(state);
+}
+
 QModelIndex QmlJSEditorWidget::outlineModelIndex()
 {
     if (!m_outlineModelIndex.isValid()) {
@@ -168,7 +198,7 @@ static void appendExtraSelectionsForMessages(
         const QList<DiagnosticMessage> &messages,
         const QTextDocument *document)
 {
-    foreach (const DiagnosticMessage &d, messages) {
+    for (const DiagnosticMessage &d : messages) {
         const int line = d.loc.startLine;
         const int column = qMax(1U, d.loc.startColumn);
 
@@ -187,7 +217,7 @@ static void appendExtraSelectionsForMessages(
             sel.cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, d.loc.length);
         }
 
-        const auto fontSettings = TextEditor::TextEditorSettings::instance()->fontSettings();
+        const auto fontSettings = TextEditor::TextEditorSettings::fontSettings();
 
         if (d.isWarning())
             sel.format = fontSettings.toTextCharFormat(TextEditor::C_WARNING);
@@ -214,10 +244,31 @@ void QmlJSEditorWidget::updateCodeWarnings(Document::Ptr doc)
     }
 }
 
-void QmlJSEditorWidget::modificationChanged(bool changed)
+void QmlJSEditorWidget::foldAuxiliaryData()
+{
+    QTextDocument *doc = document();
+    auto documentLayout = qobject_cast<TextDocumentLayout*>(doc->documentLayout());
+    QTC_ASSERT(documentLayout, return);
+    QTextBlock block = doc->lastBlock();
+
+    while (block.isValid() && block.isVisible()) {
+        if (TextBlockUserData::canFold(block) && block.next().isVisible()) {
+            const QString trimmedText = block.text().trimmed();
+            if (trimmedText.startsWith("/*##^##")) {
+                TextBlockUserData::doFoldOrUnfold(block, false);
+                documentLayout->requestUpdate();
+                documentLayout->emitDocumentSizeChanged();
+                break;
+            }
+        }
+        block = block.previous();
+    }
+}
+
+void QmlJSEditorWidget::updateModificationChange(bool changed)
 {
     if (!changed && m_modelManager)
-        m_modelManager->fileChangedOnDisk(textDocument()->filePath().toString());
+        m_modelManager->fileChangedOnDisk(textDocument()->filePath());
 }
 
 bool QmlJSEditorWidget::isOutlineCursorChangesBlocked()
@@ -227,8 +278,10 @@ bool QmlJSEditorWidget::isOutlineCursorChangesBlocked()
 
 void QmlJSEditorWidget::jumpToOutlineElement(int /*index*/)
 {
+    if (!m_outlineCombo)
+        return;
     QModelIndex index = m_outlineCombo->view()->currentIndex();
-    AST::SourceLocation location = m_qmlJsEditorDocument->outlineModel()->sourceLocation(index);
+    SourceLocation location = m_qmlJsEditorDocument->outlineModel()->sourceLocation(index);
 
     if (!location.isValid())
         return;
@@ -245,6 +298,8 @@ void QmlJSEditorWidget::jumpToOutlineElement(int /*index*/)
 
 void QmlJSEditorWidget::updateOutlineIndexNow()
 {
+    if (!m_outlineCombo)
+        return;
     if (!m_qmlJsEditorDocument->outlineModel()->document())
         return;
 
@@ -266,12 +321,6 @@ void QmlJSEditorWidget::updateOutlineIndexNow()
         m_outlineCombo->setRootModelIndex(QModelIndex());
     }
 }
-} // namespace Internal
-} // namespace QmlJSEditor
-
-
-namespace QmlJSEditor {
-namespace Internal {
 
 void QmlJSEditorWidget::updateContextPane()
 {
@@ -286,8 +335,7 @@ void QmlJSEditorWidget::updateContextPane()
 
         if (m_contextPane->isAvailable(this, info.document, newNode) &&
             !m_contextPane->widget()->isVisible()) {
-            QList<RefactorMarker> markers = RefactorMarker::filterOutType(
-                refactorMarkers(), Constants::QT_QUICK_TOOLBAR_MARKER_ID);
+            RefactorMarkers markers;
             if (UiObjectMember *m = newNode->uiObjectMemberCast()) {
                 const int start = qualifiedTypeNameId(m)->identifierToken.begin();
                 for (UiQualifiedId *q = qualifiedTypeNameId(m); q; q = q->next) {
@@ -298,8 +346,8 @@ void QmlJSEditorWidget::updateContextPane()
                             QTextCursor tc(document());
                             tc.setPosition(end);
                             marker.cursor = tc;
-                            marker.tooltip = tr("Show Qt Quick ToolBar");
-                            marker.type = Constants::QT_QUICK_TOOLBAR_MARKER_ID;
+                            marker.tooltip = Tr::tr("Show Qt Quick ToolBar");
+                            marker.type = QT_QUICK_TOOLBAR_MARKER_ID;
                             marker.callback = [this](TextEditorWidget *) {
                                 showContextPane();
                             };
@@ -308,10 +356,9 @@ void QmlJSEditorWidget::updateContextPane()
                     }
                 }
             }
-            setRefactorMarkers(markers);
+            setRefactorMarkers(markers, QT_QUICK_TOOLBAR_MARKER_ID);
         } else if (oldNode != newNode) {
-            setRefactorMarkers(RefactorMarker::filterOutType(
-                refactorMarkers(), Constants::QT_QUICK_TOOLBAR_MARKER_ID));
+            clearRefactorMarkers(QT_QUICK_TOOLBAR_MARKER_ID);
         }
         m_oldCursorPosition = position();
 
@@ -331,8 +378,14 @@ void QmlJSEditorWidget::updateUses()
         return;
 
     QList<QTextEdit::ExtraSelection> selections;
-    foreach (const AST::SourceLocation &loc,
-             m_qmlJsEditorDocument->semanticInfo().idLocations.value(wordUnderCursor())) {
+
+    // code model may present the locations not in a document order
+    const QList<SourceLocation> locations = Utils::sorted(
+                m_qmlJsEditorDocument->semanticInfo().idLocations.value(wordUnderCursor()),
+                [](const SourceLocation &lhs, const SourceLocation &rhs) {
+        return lhs.begin() < rhs.begin();
+    });
+    for (const SourceLocation &loc : locations) {
         if (! loc.isValid())
             continue;
 
@@ -369,7 +422,7 @@ protected:
     {
         UiQualifiedId *id = qualifiedTypeNameId(member);
         if (id) {
-            const QStringRef &name = id->name;
+            QStringView name = id->name;
             if (!name.isEmpty() && name.at(0).isUpper())
                 return true;
         }
@@ -387,7 +440,7 @@ protected:
             else if (script->qualifiedId->next)
                 return false;
 
-            const QStringRef &propertyName = script->qualifiedId->name;
+            QStringView propertyName = script->qualifiedId->name;
 
             if (propertyName == QLatin1String("id"))
                 return true;
@@ -431,6 +484,11 @@ protected:
             }
         }
     }
+
+    void throwRecursionDepthError() override
+    {
+        qWarning("Warning: Hit maximum recursion depth visiting AST in SelectedElement");
+    }
 };
 
 void QmlJSEditorWidget::setSelectedElements()
@@ -460,10 +518,10 @@ void QmlJSEditorWidget::setSelectedElements()
 
     if (m_qmlJsEditorDocument->semanticInfo().isValid()) {
         SelectedElement selectedMembers;
-        QList<UiObjectMember *> members
-                = selectedMembers(m_qmlJsEditorDocument->semanticInfo().document, startPos, endPos);
+        const QList<UiObjectMember *> members
+            = selectedMembers(m_qmlJsEditorDocument->semanticInfo().document, startPos, endPos);
         if (!members.isEmpty()) {
-            foreach (UiObjectMember *m, members) {
+            for (UiObjectMember *m : members) {
                 offsets << m;
             }
         }
@@ -504,7 +562,7 @@ void QmlJSEditorWidget::createToolBar()
 
     auto itemDelegate = new Utils::AnnotatedItemDelegate(this);
     itemDelegate->setDelimiter(QLatin1String(" "));
-    itemDelegate->setAnnotationRole(QmlOutlineModel::AnnotationRole);
+    itemDelegate->setAnnotationRole(Internal::QmlOutlineModel::AnnotationRole);
     treeView->setItemDelegateForColumn(0, itemDelegate);
 
     treeView->header()->hide();
@@ -520,15 +578,24 @@ void QmlJSEditorWidget::createToolBar()
     policy.setHorizontalPolicy(QSizePolicy::Expanding);
     m_outlineCombo->setSizePolicy(policy);
 
-    connect(m_outlineCombo, QOverload<int>::of(&QComboBox::activated),
+    connect(m_outlineCombo, &QComboBox::activated,
             this, &QmlJSEditorWidget::jumpToOutlineElement);
-    connect(m_qmlJsEditorDocument->outlineModel(), &QmlOutlineModel::updated,
+    connect(m_qmlJsEditorDocument->outlineModel(), &Internal::QmlOutlineModel::updated,
             static_cast<QTreeView *>(m_outlineCombo->view()), &QTreeView::expandAll);
 
     connect(this, &QmlJSEditorWidget::cursorPositionChanged,
             &m_updateOutlineIndexTimer, QOverload<>::of(&QTimer::start));
 
-    insertExtraToolBarWidget(TextEditorWidget::Left, m_outlineCombo);
+    setToolbarOutline(m_outlineCombo);
+}
+
+void QmlJSEditorWidget::updateOutline(QWidget *newOutline)
+{
+    if (!newOutline) {
+        createToolBar();
+    } else if (newOutline != m_outlineCombo){
+        m_outlineCombo = nullptr;
+    }
 }
 
 class CodeModelInspector : public MemberProcessor
@@ -556,24 +623,24 @@ public:
         *m_stream << m_indent;
         if (!propertyInfo.isWriteable())
             *m_stream << "readonly ";
-        *m_stream << "property " << type << " " << name << endl;
+        *m_stream << "property " << type << " " << name << '\n';
 
         return true;
     }
     bool processSignal(const QString &name, const Value *value) override
     {
-        *m_stream << m_indent << "signal " << name << stringifyFunctionParameters(value) << endl;
+        *m_stream << m_indent << "signal " << name << stringifyFunctionParameters(value) << '\n';
         return true;
     }
     bool processSlot(const QString &name, const Value *value) override
     {
-        *m_stream << m_indent << "function " << name << stringifyFunctionParameters(value) << endl;
+        *m_stream << m_indent << "function " << name << stringifyFunctionParameters(value) << '\n';
         return true;
     }
     bool processGeneratedSlot(const QString &name, const Value *value) override
     {
         *m_stream << m_indent << "/*generated*/ function " << name
-                  << stringifyFunctionParameters(value) << endl;
+                  << stringifyFunctionParameters(value) << '\n';
         return true;
     }
 
@@ -630,33 +697,33 @@ static QString inspectCppComponent(const CppComponentValue *cppValue)
     if (superClassName.isEmpty())
         superClassName = cppValue->metaObject()->className();
 
-    bufWriter << "import QtQuick " << cppValue->importVersion().toString() << endl
+    bufWriter << "import QtQuick " << cppValue->importVersion().toString() << '\n'
               << "// " << cppValue->metaObject()->className()
               << " imported as " << cppValue->moduleName()  << " "
-              << cppValue->importVersion().toString() << endl
-              << endl
-              << superClassName << " {" << endl;
+              << cppValue->importVersion().toString() << '\n'
+              << '\n'
+              << superClassName << " {" << '\n';
 
     CodeModelInspector insp(cppValue, &bufWriter);
     cppValue->processMembers(&insp);
 
-    bufWriter << endl;
+    bufWriter << '\n';
     const int enumeratorCount = cppValue->metaObject()->enumeratorCount();
     for (int index = cppValue->metaObject()->enumeratorOffset(); index < enumeratorCount; ++index) {
         LanguageUtils::FakeMetaEnum enumerator = cppValue->metaObject()->enumerator(index);
-        bufWriter << "    enum " << enumerator.name() << " {" << endl;
+        bufWriter << "    enum " << enumerator.name() << " {" << '\n';
         const QStringList keys = enumerator.keys();
         const int keysCount = keys.size();
         for (int i = 0; i < keysCount; ++i) {
             bufWriter << "        " << keys.at(i);
             if (i != keysCount - 1)
                 bufWriter << ',';
-            bufWriter << endl;
+            bufWriter << '\n';
         }
-        bufWriter << "    }" << endl;
+        bufWriter << "    }" << '\n';
     }
 
-    bufWriter << "}" << endl;
+    bufWriter << "}" << '\n';
     return result;
 }
 
@@ -671,17 +738,17 @@ void QmlJSEditorWidget::inspectElementUnderCursor() const
 
     const CppComponentValue *cppValue = findCppComponentToInspect(semanticInfo, cursorPosition);
     if (!cppValue) {
-        QString title = tr("Code Model Not Available");
-        const QString documentId = Constants::QML_JS_EDITOR_PLUGIN + QStringLiteral(".NothingToShow");
+        QString title = Tr::tr("Code Model Not Available");
+        const QString documentId = QML_JS_EDITOR_PLUGIN + QStringLiteral(".NothingToShow");
         EditorManager::openEditorWithContents(Core::Constants::K_DEFAULT_TEXT_EDITOR_ID, &title,
-                                              tr("Code model not available.").toUtf8(), documentId,
+                                              Tr::tr("Code model not available.").toUtf8(), documentId,
                                               EditorManager::IgnoreNavigationHistory);
         return;
     }
 
-    QString title = tr("Code Model of %1").arg(cppValue->metaObject()->className());
-    const QString documentId = Constants::QML_JS_EDITOR_PLUGIN + QStringLiteral(".Class.")
-            + cppValue->metaObject()->className();
+    QString title = Tr::tr("Code Model of %1").arg(cppValue->metaObject()->className());
+    const QString documentId = QML_JS_EDITOR_PLUGIN + QStringLiteral(".Class.")
+                               + cppValue->metaObject()->className();
     IEditor *outputEditor = EditorManager::openEditorWithContents(
                 Core::Constants::K_DEFAULT_TEXT_EDITOR_ID, &title, QByteArray(),
                 documentId, EditorManager::IgnoreNavigationHistory);
@@ -695,17 +762,28 @@ void QmlJSEditorWidget::inspectElementUnderCursor() const
 
     widget->setReadOnly(true);
     widget->textDocument()->setTemporary(true);
-    widget->textDocument()->setSyntaxHighlighter(new QmlJSHighlighter(widget->document()));
+    widget->textDocument()->resetSyntaxHighlighter([] { return new QmlJSHighlighter(); });
 
     const QString buf = inspectCppComponent(cppValue);
     widget->textDocument()->setPlainText(buf);
 }
 
 void QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
-                                   Utils::ProcessLinkCallback &&processLinkCallback,
-                                   bool /*resolveTarget*/,
+                                   const Utils::LinkHandler &processLinkCallback,
+                                   bool resolveTarget,
                                    bool /*inNextSplit*/)
 {
+    if (auto client = getQmllsClient(ProjectExplorer::ProjectManager::startupProject(),
+                                     textDocument()->filePath(),
+                                     DefaultFunctionality)) {
+        client->findLinkAt(textDocument(),
+                           cursor,
+                           processLinkCallback,
+                           resolveTarget,
+                           LanguageClient::LinkTarget::SymbolDef);
+        return;
+    }
+
     const SemanticInfo semanticInfo = m_qmlJsEditorDocument->semanticInfo();
     if (! semanticInfo.isValid())
         return processLinkCallback(Utils::Link());
@@ -717,9 +795,11 @@ void QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
 
     if (auto importAst = cast<const AST::UiImport *>(node)) {
         // if it's a file import, link to the file
-        foreach (const ImportInfo &import, semanticInfo.document->bind()->imports()) {
+        const QList<ImportInfo> imports = semanticInfo.document->bind()->imports();
+        for (const ImportInfo &import : imports) {
             if (import.ast() == importAst && import.type() == ImportType::File) {
-                Utils::Link link(import.path());
+                Utils::Link link(
+                    m_modelManager->fileToSource(FilePath::fromString(import.path())));
                 link.linkTextStart = importAst->firstSourceLocation().begin();
                 link.linkTextEnd = importAst->lastSourceLocation().end();
                 processLinkCallback(Utils::Link());
@@ -730,22 +810,54 @@ void QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
         return;
     }
 
+    const ProjectExplorer::Project * const project = ProjectExplorer::ProjectTree::currentProject();
+    ProjectExplorer::ProjectNode* projectRootNode = nullptr;
+    if (project) {
+        projectRootNode = project->rootProjectNode();
+    }
+
     // string literals that could refer to a file link to them
     if (auto literal = cast<const StringLiteral *>(node)) {
         const QString &text = literal->value.toString();
-        Utils::Link link;
-        link.linkTextStart = literal->literalToken.begin();
-        link.linkTextEnd = literal->literalToken.end();
-        if (semanticInfo.snapshot.document(text)) {
-            link.targetFileName = text;
+        if (text.startsWith("qrc:/")) {
+            if (projectRootNode) {
+                const ProjectExplorer::Node * const nodeForPath = projectRootNode->findNode(
+                            [qrcPath = text.mid(text.indexOf(':') + 1)](ProjectExplorer::Node *n) {
+                    if (!n->asFileNode())
+                        return false;
+                    const auto qrcNode = dynamic_cast<ProjectExplorer::ResourceFileNode *>(n);
+                    return qrcNode && qrcNode->qrcPath() == qrcPath;
+                });
+                if (nodeForPath) {
+                    Link link(nodeForPath->filePath());
+                    link.linkTextStart = literal->firstSourceLocation().begin();
+                    link.linkTextEnd = literal->lastSourceLocation().end();
+                    processLinkCallback(link);
+                    return;
+                }
+            }
+        }
+
+        if (text.startsWith("https:/") || text.startsWith("http:/")) {
+            Link link = Link::fromString(text);
+            link.linkTextStart = literal->literalToken.begin();
+            link.linkTextEnd = literal->literalToken.end();
             processLinkCallback(link);
             return;
         }
-        const QString relative = QString::fromLatin1("%1/%2").arg(
-                    semanticInfo.document->path(),
-                    text);
-        if (semanticInfo.snapshot.document(relative)) {
-            link.targetFileName = relative;
+
+        Utils::Link link;
+        link.linkTextStart = literal->literalToken.begin();
+        link.linkTextEnd = literal->literalToken.end();
+        Utils::FilePath targetFilePath = Utils::FilePath::fromUserInput(text);
+        if (semanticInfo.snapshot.document(targetFilePath)) {
+            link.targetFilePath = targetFilePath;
+            processLinkCallback(link);
+            return;
+        }
+        const Utils::FilePath relative = semanticInfo.document->path().pathAppended(text);
+        if (relative.exists()) {
+            link.targetFilePath = m_modelManager->fileToSource(relative);
             processLinkCallback(link);
             return;
         }
@@ -755,32 +867,61 @@ void QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
     Evaluate evaluator(&scopeChain);
     const Value *value = evaluator.reference(node);
 
-    QString fileName;
+    Utils::FilePath fileName;
     int line = 0, column = 0;
 
     if (! (value && value->getSourceLocation(&fileName, &line, &column)))
         return processLinkCallback(Utils::Link());
 
     Utils::Link link;
-    link.targetFileName = fileName;
+    link.targetFilePath = m_modelManager->fileToSource(fileName);
     link.targetLine = line;
     link.targetColumn = column - 1; // adjust the column
 
+    auto processPotentialCppLink = [&]() -> bool {
+        if (!value->asCppComponentValue() || !projectRootNode) {
+            processLinkCallback(link);
+            return true;
+        }
+
+        const ProjectExplorer::Node * const nodeForPath = projectRootNode->findNode(
+            [&fileName](ProjectExplorer::Node *n) {
+                const auto fileNode = n->asFileNode();
+                if (!fileNode)
+                    return false;
+                Utils::FilePath filePath = n->filePath();
+                return filePath.endsWith(fileName.toUserOutput());
+            });
+        if (nodeForPath) {
+            link.targetFilePath = nodeForPath->filePath();
+            processLinkCallback(link);
+            return true;
+        }
+
+        // else we will process an empty link below to avoid an error dialog
+        return false;
+    };
+
     if (auto q = AST::cast<const AST::UiQualifiedId *>(node)) {
         for (const AST::UiQualifiedId *tail = q; tail; tail = tail->next) {
-            if (! tail->next && cursorPosition <= tail->identifierToken.end()) {
-                link.linkTextStart = tail->identifierToken.begin();
-                link.linkTextEnd = tail->identifierToken.end();
-                processLinkCallback(link);
+            if (tail->next || !(cursorPosition <= tail->identifierToken.end())) {
+                continue;
+            }
+
+            link.linkTextStart = tail->identifierToken.begin();
+            link.linkTextEnd = tail->identifierToken.end();
+
+            if (processPotentialCppLink()) {
                 return;
             }
         }
-
     } else if (auto id = AST::cast<const AST::IdentifierExpression *>(node)) {
         link.linkTextStart = id->firstSourceLocation().begin();
         link.linkTextEnd = id->lastSourceLocation().end();
-        processLinkCallback(link);
-        return;
+
+        if (processPotentialCppLink()) {
+            return;
+        }
 
     } else if (auto mem = AST::cast<const AST::FieldMemberExpression *>(node)) {
         link.linkTextStart = mem->lastSourceLocation().begin();
@@ -794,12 +935,28 @@ void QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
 
 void QmlJSEditorWidget::findUsages()
 {
-    m_findReferences->findUsages(textDocument()->filePath().toString(), textCursor().position());
+    const Utils::FilePath fileName = textDocument()->filePath();
+
+    if (auto client = getQmllsClient(ProjectExplorer::ProjectManager::startupProject(),
+                                     fileName, DefaultFunctionality)) {
+        client->symbolSupport().findUsages(textDocument(), textCursor());
+    } else {
+        const int offset = textCursor().position();
+        m_findReferences->findUsages(fileName, offset);
+    }
 }
 
-void QmlJSEditorWidget::renameUsages()
+void QmlJSEditorWidget::renameSymbolUnderCursor()
 {
-    m_findReferences->renameUsages(textDocument()->filePath().toString(), textCursor().position());
+    const Utils::FilePath fileName = textDocument()->filePath();
+
+    if (auto client = getQmllsClient(ProjectExplorer::ProjectManager::startupProject(),
+                                     fileName, DefaultFunctionality)) {
+        client->symbolSupport().renameSymbol(textDocument(), textCursor(), QString());
+    } else {
+        const int offset = textCursor().position();
+        m_findReferences->renameUsages(fileName, offset);
+    }
 }
 
 void QmlJSEditorWidget::showContextPane()
@@ -812,8 +969,7 @@ void QmlJSEditorWidget::showContextPane()
                              &scopeChain,
                              newNode, false, true);
         m_oldCursorPosition = position();
-        setRefactorMarkers(RefactorMarker::filterOutType(
-            refactorMarkers(), Constants::QT_QUICK_TOOLBAR_MARKER_ID));
+        clearRefactorMarkers(QT_QUICK_TOOLBAR_MARKER_ID);
     }
 }
 
@@ -821,14 +977,14 @@ void QmlJSEditorWidget::contextMenuEvent(QContextMenuEvent *e)
 {
     QPointer<QMenu> menu(new QMenu(this));
 
-    QMenu *refactoringMenu = new QMenu(tr("Refactoring"), menu);
+    QMenu *refactoringMenu = new QMenu(Tr::tr("Refactoring"), menu);
 
     if (!m_qmlJsEditorDocument->isSemanticInfoOutdated()) {
-        AssistInterface *interface = createAssistInterface(QuickFix, ExplicitlyInvoked);
+        std::unique_ptr<AssistInterface> interface = createAssistInterface(QuickFix, ExplicitlyInvoked);
         if (interface) {
             QScopedPointer<IAssistProcessor> processor(
-                        QmlJSEditorPlugin::quickFixAssistProvider()->createProcessor());
-            QScopedPointer<IAssistProposal> proposal(processor->perform(interface));
+                Internal::quickFixAssistProvider()->createProcessor(interface.get()));
+            QScopedPointer<IAssistProposal> proposal(processor->start(std::move(interface)));
             if (!proposal.isNull()) {
                 GenericProposalModelPtr model = proposal->model().staticCast<GenericProposalModel>();
                 for (int index = 0; index < model->size(); ++index) {
@@ -845,7 +1001,8 @@ void QmlJSEditorWidget::contextMenuEvent(QContextMenuEvent *e)
 
     if (ActionContainer *mcontext = ActionManager::actionContainer(Constants::M_CONTEXT)) {
         QMenu *contextMenu = mcontext->menu();
-        foreach (QAction *action, contextMenu->actions()) {
+        const QList<QAction *> actions = contextMenu->actions();
+        for (QAction *action : actions) {
             menu->addAction(action);
             if (action->objectName() == QLatin1String(Constants::M_REFACTORING_MENU_INSERTION_POINT))
                 menu->addMenu(refactoringMenu);
@@ -936,11 +1093,11 @@ QModelIndex QmlJSEditorWidget::indexForPosition(unsigned cursorPosition, const Q
 {
     QModelIndex lastIndex = rootIndex;
 
-    QmlOutlineModel *model = m_qmlJsEditorDocument->outlineModel();
+    Internal::QmlOutlineModel *model = m_qmlJsEditorDocument->outlineModel();
     const int rowCount = model->rowCount(rootIndex);
     for (int i = 0; i < rowCount; ++i) {
         QModelIndex childIndex = model->index(i, 0, rootIndex);
-        AST::SourceLocation location = model->sourceLocation(childIndex);
+        SourceLocation location = model->sourceLocation(childIndex);
 
         if ((cursorPosition >= location.offset)
               && (cursorPosition <= location.offset + location.length)) {
@@ -965,20 +1122,18 @@ bool QmlJSEditorWidget::hideContextPane()
     return b;
 }
 
-AssistInterface *QmlJSEditorWidget::createAssistInterface(
+std::unique_ptr<AssistInterface> QmlJSEditorWidget::createAssistInterface(
     AssistKind assistKind,
     AssistReason reason) const
 {
     if (assistKind == Completion) {
-        return new QmlJSCompletionAssistInterface(document(),
-                                                  position(),
-                                                  textDocument()->filePath().toString(),
-                                                  reason,
-                                                  m_qmlJsEditorDocument->semanticInfo());
+        return std::make_unique<QmlJSCompletionAssistInterface>(
+            textCursor(), textDocument()->filePath(), reason, m_qmlJsEditorDocument->semanticInfo());
     } else if (assistKind == QuickFix) {
-        return new QmlJSQuickFixAssistInterface(const_cast<QmlJSEditorWidget *>(this), reason);
+        return std::make_unique<Internal::QmlJSQuickFixAssistInterface>(
+            const_cast<QmlJSEditorWidget *>(this), reason);
     }
-    return nullptr;
+    return TextEditorWidget::createAssistInterface(assistKind, reason);
 }
 
 QString QmlJSEditorWidget::foldReplacementText(const QTextBlock &block) const
@@ -1007,37 +1162,39 @@ QmlJSEditor::QmlJSEditor()
     addContext(ProjectExplorer::Constants::QMLJS_LANGUAGE_ID);
 }
 
+QmlJSEditorDocument *QmlJSEditor::qmlJSDocument() const
+{
+    return qobject_cast<QmlJSEditorDocument *>(document());
+}
+
 bool QmlJSEditor::isDesignModePreferred() const
 {
 
-    bool alwaysPreferDesignMode = false;
-    // always prefer design mode for .ui.qml files
-    if (textDocument() && textDocument()->mimeType() == QLatin1String(QmlJSTools::Constants::QMLUI_MIMETYPE))
-        alwaysPreferDesignMode = true;
-
     // stay in design mode if we are there
-    Id mode = ModeManager::currentModeId();
-    return alwaysPreferDesignMode || mode == Core::Constants::MODE_DESIGN;
+    const Id mode = ModeManager::currentModeId();
+    return qmlJSDocument()->isDesignModePreferred() || mode == Core::Constants::MODE_DESIGN;
 }
 
-
 //
 // QmlJSEditorFactory
 //
 
 QmlJSEditorFactory::QmlJSEditorFactory()
+    : QmlJSEditorFactory(Constants::C_QMLJSEDITOR_ID)
+{}
+
+QmlJSEditorFactory::QmlJSEditorFactory(Utils::Id _id)
 {
-    setId(Constants::C_QMLJSEDITOR_ID);
-    setDisplayName(QCoreApplication::translate("OpenWith::Editors", Constants::C_QMLJSEDITOR_DISPLAY_NAME));
+    setId(_id);
+    setDisplayName(::Core::Tr::tr("QMLJS Editor"));
 
-    addMimeType(QmlJSTools::Constants::QML_MIMETYPE);
-    addMimeType(QmlJSTools::Constants::QMLUI_MIMETYPE);
-    addMimeType(QmlJSTools::Constants::QMLPROJECT_MIMETYPE);
-    addMimeType(QmlJSTools::Constants::QBS_MIMETYPE);
-    addMimeType(QmlJSTools::Constants::QMLTYPES_MIMETYPE);
-    addMimeType(QmlJSTools::Constants::JS_MIMETYPE);
+    using namespace Utils::Constants;
+    addMimeType(QML_MIMETYPE);
+    addMimeType(QMLPROJECT_MIMETYPE);
+    addMimeType(QMLTYPES_MIMETYPE);
+    addMimeType(JS_MIMETYPE);
 
-    setDocumentCreator([]() { return new QmlJSEditorDocument; });
+    setDocumentCreator([this]() { return new QmlJSEditorDocument(id()); });
     setEditorWidgetCreator([]() { return new QmlJSEditorWidget; });
     setEditorCreator([]() { return new QmlJSEditor; });
     setAutoCompleterCreator([]() { return new AutoCompleter; });
@@ -1046,20 +1203,63 @@ QmlJSEditorFactory::QmlJSEditorFactory()
     setCodeFoldingSupported(true);
 
     addHoverHandler(new QmlJSHoverHandler);
+    addHoverHandler(new ColorPreviewHoverHandler);
     setCompletionAssistProvider(new QmlJSCompletionAssistProvider);
 
-    setEditorActionHandlers(TextEditorActionHandler::Format
-        | TextEditorActionHandler::UnCommentSelection
-        | TextEditorActionHandler::UnCollapseAll
-                            | TextEditorActionHandler::FollowSymbolUnderCursor);
+    setOptionalActionMask(OptionalActions::Format
+                            | OptionalActions::UnCommentSelection
+                            | OptionalActions::UnCollapseAll
+                            | OptionalActions::FollowSymbolUnderCursor
+                            | OptionalActions::RenameSymbol
+                            | OptionalActions::FindUsage);
 }
 
-void QmlJSEditorFactory::decorateEditor(TextEditorWidget *editor)
+static void decorateEditor(TextEditorWidget *editor)
 {
-    editor->textDocument()->setSyntaxHighlighter(new QmlJSHighlighter);
-    editor->textDocument()->setIndenter(new Indenter(editor->textDocument()->document()));
+    editor->textDocument()->resetSyntaxHighlighter([] { return new QmlJSHighlighter(); });
+    editor->textDocument()->setIndenter(createQmlJsIndenter(editor->textDocument()->document()));
     editor->setAutoCompleter(new AutoCompleter);
 }
 
+namespace Internal {
+
+void inspectElement()
+{
+    if (auto widget = qobject_cast<QmlJSEditorWidget *>(EditorManager::currentEditor()->widget()))
+        widget->inspectElementUnderCursor();
+}
+
+void showContextPane()
+{
+    if (auto editor = qobject_cast<QmlJSEditorWidget*>(EditorManager::currentEditor()->widget()))
+        editor->showContextPane();
+}
+
+void setupQmlJSEditor()
+{
+    static QmlJSEditorFactory theQmlJSEditorFactory;
+
+    TextEditor::SnippetProvider::registerGroup(Constants::QML_SNIPPETS_GROUP_ID,
+                                               Tr::tr("QML", "SnippetProvider"),
+                                               &decorateEditor);
+
+}
+
 } // namespace Internal
+
+void setQdsSettingVisible(bool visible)
+{
+    Internal::settings().qdsCommand.setVisible(visible);
+}
+
+FilePath qdsCommand()
+{
+    const FilePath command = Internal::settings().qdsCommand.effectiveBinary();
+    if (command.isEmpty())
+        return Internal::settings().defaultQdsCommand();
+    return command;
+}
+
+// namespace Internal
+
 } // namespace QmlJSEditor
